% Versuch 2_2
% C. Roppel, HS Schmalkalden
% MATLAB- und Scilab-bungen zum Buch "Grundlagen der Nachrichtentechnik", Hanser-Verlag, 2023
% Getestet mit MATLAB R2020a
%%
clear
clc
close all

%% Parameter
alpha = 0.5;    % Roll-off-Faktor
os = 8;         % Samples pro Symbol (Oversampling Ratio)
dly_rc = 3;     % Verzgerung des Wurzel-Kosinus-roll-off-Filters
nSym = 1000;    % Anzahl Symbole
m = 4;          % m-wertige Symbole
k = log2(m);    % k Bit pro Symbol
lambda = pi/4;  % QPSK Phasenoffset
EbN0 = 6;       % Eb/N0 in dB

%% Wurzel-Kosinus-roll-off-Filter
psrc = rcosdesign(alpha, 2*dly_rc, os, 'sqrt');

%% QPSK-Modulator und Pulsformfilter
rng(1234);  % Initialisiere Zufallsgenerator
symbols = randi([0 (m - 1)], nSym, 1);  % Spaltenvektor der Lnge nSym mit Zufallszahlen im Bereich 0 bis m-1

xmod = pskmod(symbols, m, lambda);

xmod = upsample(xmod, os);  % Einfgen von os - 1 Nullen zwischen den Symbolen
x = conv(psrc, xmod);  % Faltung (convolution)
x = x(dly_rc*os + 1:end - dly_rc*os);  % Entfernen der ersten und letzten Werte

%% AWGN-Kanal
snr = EbN0 + 10*log10(k) - 10*log10(os);  % Strabstand in dB
sigPwr = 0.5*mean(abs(x).^2);  % Signalleistung
noisePwr = sigPwr/(10^(snr/10));  % Rauschleistung
rng(1234); 
noise_i = randn(length(x), 1)*sqrt(noisePwr);
rng(4321); 
noise_q = randn(length(x), 1)*sqrt(noisePwr);
y = x + noise_i + j*noise_q;  % Signal + Rauschen

figure; 
periodogram(y, [], [], os, 'centered');

%% Signalangepasstes Filter und QPSK-Demodulator
ye = conv(psrc, y);
ye = ye(dly_rc*os + 1:end - dly_rc*os);

ye = downsample(ye, os);

scatterplot(ye);

ysym = pskdemod(ye, m, lambda);

% Bestimmung der Symbolfehlerhufigkeit
[nerror, ser] = symerr(symbols, ysym);
if m <= 4
    serTheo = erfc(sqrt(10^(EbN0/10)));
else
    serTheo = erfc(sqrt(log2(m)*10^(EbN0/10))*sin(pi/m));
end
fprintf('Eb/N0: %f dB  Ps: %f  Symbolfehler: %d  Symbolfehlerhufigkeit: %f\n', EbN0, serTheo, nerror, ser);

