% Versuch 3_2
% C. Roppel, HS Schmalkalden
% MATLAB- und Scilab-bungen zum Buch "Grundlagen der Nachrichtentechnik", Hanser-Verlag, 2023
% Getestet mit MATLAB R2020a
%%
clear
clc
close all

%% Parameter
alpha = 0.5;    % Roll-off-Faktor
os = 8;         % Samples pro Symbol (Oversampling Ratio)
dly_rc = 3;     % Verzgerung des Wurzel-Kosinus-roll-off-Filters
nSym = 1000;    % Anzahl Symbole
m = 4;          % m-wertige Symbole
k = log2(m);    % k Bit pro Symbol
lambda = pi/4;  % QPSK Phasenoffset

%% Wurzel-Kosinus-roll-off-Filter
psrc = rcosdesign(alpha, 2*dly_rc, os, 'sqrt');

%% QPSK-Modulator und Pulsformfilter
rng(1234);  % Initialisiere Zufallsgenerator
symbols = randi([0 (m - 1)], nSym, 1);  % Spaltenvektor der Lnge nSym mit Zufallszahlen im Bereich 0 bis m-1

xmod = pskmod(symbols, m, lambda);  % Modulation

xmod = upsample(xmod, os);  % Einfgen von os - 1 Nullen zwischen den Symbolen
x = conv(psrc, xmod);  % Faltung (convolution)
x = x(dly_rc*os + 1:end - dly_rc*os);  % Entfernen der ersten und letzten Werte

%% Mehrwegekanal
dt = 2;   % Laufzeitdifferenz (in Samples)
a = 0.8;  % Dmpfung des indirekten Pfades
b = 1;    % Produkt f_c * t_0

% Impulsantwort
hKanal = zeros(1, dt + 1);
hKanal(1) = 1;
hKanal(dt + 1) = a*exp(-j*2*pi*b);

% Faltung mit der Kanalimpulsantwort
r = conv(hKanal, x);

figure; 
periodogram(r, [], [], os, 'centered');

%% T/2-Entzerrer
Ne = 33;                   % Anzahl der Koeffizienten
dly_eq = round((Ne-1)/2);  % Verzgerung des Entzerrers

% Zu entzerrender Impuls g(n)
gn = conv(psrc, hKanal);
gn = downsample(gn, os/2);

% Rauschleistung
var_r = 1e-6;
% Faltungsmatrix mit m + 1 + N Zeilen und m + 1 Spalten
gn = gn(:);
F = toeplitz([gn; zeros(Ne-1, 1)], [gn(1) zeros(1, Ne-1)]);
% F hat z Zeilen und s Spalten, Weglassen jeder zweiten Zeile
[z, s] = size(F);
F(2:2:z, :) = [];
% Ideales Ausgangssignal
[z, s] = size(F)
yid = zeros(z, 1);
yid(round(z/2)) = 1;
% Berechnung der Koeffizienten
I = diag(ones(1, s));
en = (F'*F + var_r*I)\F'*yid;

figure;
subplot(2, 1, 1);
stem(real(gn));
ylabel('g_i(n)');
subplot(2, 1, 2);
stem(imag(gn));
ylabel('g_q(n)');
xlabel('n');

figure;
subplot(2, 1, 1);
stem(real(en));
ylabel('e_i(n)');
subplot(2, 1, 2);
stem(imag(en));
ylabel('e_q(n)');
xlabel('n');

% Eingangssignal, 2 Samples/Symbol, Entzerrung
r = downsample(r, os/2);
y = conv(en, r);
y = y(dly_eq + 1:end - dly_eq);
eyediagram(y, 4, 2);
